/*
 * Copyright (C) 1998 - 2005
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/**
 * \file   ltiInterpolationType.h
 *         Defines the enumeration type eInterpolationType used in many
 *         places to specify which kind of interpolation should be
 *         employed.
 * \author Pablo Alvarado
 * \date   21.11.2003
 *
 * $Id: ltiInterpolationType.h,v 1.1 2005/01/12 13:27:26 doerfler Exp $
 */

#ifndef _LTI_INTERPOLATION_TYPE_H_
#define _LTI_INTERPOLATION_TYPE_H_

#include "ltiIoHandler.h"


namespace lti {

  /**
   * Interpolation Type.
   *
   * This type is used to indicate how sub-pixel position should be computed.
   */
  enum eInterpolationType {
    NearestNeighborInterpolation, /**< Nearest Neighbor Interpolation */
    BilinearInterpolation,        /**< Bilinear Interpolation */
    BiquadraticInterpolation,     /**< Biquadratic Taps Interpolation */
    BicubicInterpolation          /**< Bicubic Interpolation */
  };
  
  /**
   * read function for eInterpolationType.
   *
   * @ingroup gStorable
   */
  bool read(ioHandler& handler,eInterpolationType& data);

  /**
   * write function for eInterpolationType.
   *
   * @ingroup gStorable
   */
  bool write(ioHandler& handler,const eInterpolationType& data);

  
}
#endif
